import { Type as Timestamp } from "../proto/messages/google/protobuf/Timestamp";
/**
 * Creates and returns a Protocol Buffer Timestamp object based on the current time.
 *
 * This function captures the current time and converts it to a Protocol Buffer
 * Timestamp object with seconds since epoch and nanoseconds precision.
 *
 * @returns A Protocol Buffer Timestamp object representing the current time
 *          with seconds and nanoseconds components.
 */
export declare function timestamp(): Timestamp;
/**
 * Returns the current time as an RFC 3339 formatted string suitable for
 * Protocol Buffer Timestamp JSON serialization.
 *
 * Reference: https://protobuf.dev/reference/php/api-docs/Google/Protobuf/Timestamp.html
 *
 * The format is: "{year}-{month}-{day}T{hour}:{min}:{sec}[.{frac_sec}]Z"
 * - All components except year are zero-padded to two digits
 * - Year is expressed using four digits
 * - Fractional seconds can go up to 9 digits (nanosecond precision)
 * - The "Z" suffix indicates UTC timezone
 *
 * @returns Current time in RFC 3339 format with UTC timezone (Z)
 */
export declare function timestampRfc3339(): string;
//# sourceMappingURL=timestamp.d.ts.map